@@ -15,13 +15,12 @@ from kwave.utils.tictoc import TicToc
 
 
 def save_to_disk_func(
-        kgrid: kWaveGrid, medium: kWaveMedium, source,
-        opt: SimulationOptions, auto_chunk: bool,
-        values: dotdict, flags: dotdict):
+    kgrid: kWaveGrid, medium: kWaveMedium, source, opt: SimulationOptions, auto_chunk: bool, values: dotdict, flags: dotdict
+):
     # update command line status
-    logging.log(logging.INFO, '  precomputation completed in ', scale_time(TicToc.toc()))
+    logging.log(logging.INFO, "  precomputation completed in ", scale_time(TicToc.toc()))
     TicToc.tic()
-    logging.log(logging.INFO, '  saving input files to disk...')
+    logging.log(logging.INFO, "  saving input files to disk...")
 
     # check for a binary sensor mask or cuboid corners
     # modified by Farid | disabled temporarily!
@@ -45,9 +44,16 @@ def save_to_disk_func(
         integer_variables.pml_z_size = 0
 
     grab_medium_props(integer_variables, float_variables, medium, flags.elastic_code)
-    grab_source_props(integer_variables, float_variables, source,
-                      values.u_source_pos_index, values.s_source_pos_index, values.p_source_pos_index,
-                      values.transducer_input_signal, values.delay_mask)
+    grab_source_props(
+        integer_variables,
+        float_variables,
+        source,
+        values.u_source_pos_index,
+        values.s_source_pos_index,
+        values.p_source_pos_index,
+        values.transducer_input_signal,
+        values.delay_mask,
+    )
 
     grab_sensor_props(integer_variables, kgrid.dim, values.sensor_mask_index, values.record.cuboid_corners_list)
     grab_nonuniform_grid_props(float_variables, kgrid, flags.nonuniform_grid)
@@ -57,70 +63,67 @@ def save_to_disk_func(
     # =========================================================================
 
     remove_z_dimension(float_variables, kgrid.dim)
-    save_file(opt.input_filename, integer_variables, float_variables, opt.hdf_compression_level, 
-              auto_chunk=auto_chunk)
+    save_file(opt.input_filename, integer_variables, float_variables, opt.hdf_compression_level, auto_chunk=auto_chunk)
 
     # update command line status
-    logging.log(logging.INFO, '  completed in ', scale_time(TicToc.toc()))
+    logging.log(logging.INFO, "  completed in ", scale_time(TicToc.toc()))
 
 
 def grab_integer_variables(integer_variables, kgrid, flags, medium):
     # integer variables used within the time loop for all codes
 
-    variables = dotdict({
-        'Nx': kgrid.Nx,
-        'Ny': kgrid.Ny,
-        'Nz': kgrid.Nz,
-        'Nt': kgrid.Nt,
-        'p_source_flag': flags.source_p,
-        'p0_source_flag': flags.source_p0,
-
-        'ux_source_flag': flags.source_ux,
-        'uy_source_flag': flags.source_uy,
-        'uz_source_flag': flags.source_uz,
-
-        'sxx_source_flag': flags.source_sxx,
-        'syy_source_flag': flags.source_syy,
-        'szz_source_flag': flags.source_szz,
-
-        'sxy_source_flag': flags.source_sxy,
-        'sxz_source_flag': flags.source_sxz,
-        'syz_source_flag': flags.source_syz,
-
-        'transducer_source_flag': flags.transducer_source,
-        'nonuniform_grid_flag': flags.nonuniform_grid,
-        'nonlinear_flag': medium.is_nonlinear(),
-        'absorbing_flag': None,
-        'elastic_flag': flags.elastic_code,
-        'axisymmetric_flag': flags.axisymmetric,
-
-        # create pseudonyms for the sensor flgs
-        #   0: binary mask indices
-        #   1: cuboid corners
-        'sensor_mask_type': flags.cuboid_corners
-    })
+    variables = dotdict(
+        {
+            "Nx": kgrid.Nx,
+            "Ny": kgrid.Ny,
+            "Nz": kgrid.Nz,
+            "Nt": kgrid.Nt,
+            "p_source_flag": flags.source_p,
+            "p0_source_flag": flags.source_p0,
+            "ux_source_flag": flags.source_ux,
+            "uy_source_flag": flags.source_uy,
+            "uz_source_flag": flags.source_uz,
+            "sxx_source_flag": flags.source_sxx,
+            "syy_source_flag": flags.source_syy,
+            "szz_source_flag": flags.source_szz,
+            "sxy_source_flag": flags.source_sxy,
+            "sxz_source_flag": flags.source_sxz,
+            "syz_source_flag": flags.source_syz,
+            "transducer_source_flag": flags.transducer_source,
+            "nonuniform_grid_flag": flags.nonuniform_grid,
+            "nonlinear_flag": medium.is_nonlinear(),
+            "absorbing_flag": None,
+            "elastic_flag": flags.elastic_code,
+            "axisymmetric_flag": flags.axisymmetric,
+            # create pseudonyms for the sensor flgs
+            #   0: binary mask indices
+            #   1: cuboid corners
+            "sensor_mask_type": flags.cuboid_corners,
+        }
+    )
     integer_variables.update(variables)
 
 
 def grab_pml_size(integer_variables, opt):
     # additional integer variables not used within time loop but stored directly to output file
-    integer_variables['pml_x_size'] = opt.pml_x_size
-    integer_variables['pml_y_size'] = opt.pml_y_size
-    integer_variables['pml_z_size'] = opt.pml_z_size
+    integer_variables["pml_x_size"] = opt.pml_x_size
+    integer_variables["pml_y_size"] = opt.pml_y_size
+    integer_variables["pml_z_size"] = opt.pml_z_size
 
 
 def grab_float_variables(float_variables: dotdict, kgrid, opt, values, is_elastic_code, is_axisymmetric):
     # single precision variables not used within time loop but stored directly
     # to the output file for all files
-    variables = dotdict({
-        'dx': kgrid.dx,
-        'dy': kgrid.dy,
-        'dz': kgrid.dz,
-
-        'pml_x_alpha': opt.pml_x_alpha,
-        'pml_y_alpha': opt.pml_y_alpha,
-        'pml_z_alpha': opt.pml_z_alpha
-    })
+    variables = dotdict(
+        {
+            "dx": kgrid.dx,
+            "dy": kgrid.dy,
+            "dz": kgrid.dz,
+            "pml_x_alpha": opt.pml_x_alpha,
+            "pml_y_alpha": opt.pml_y_alpha,
+            "pml_z_alpha": opt.pml_z_alpha,
+        }
+    )
     float_variables.update(variables)
 
     if is_elastic_code:  # pragma: no cover
@@ -129,29 +132,29 @@ def grab_float_variables(float_variables: dotdict, kgrid, opt, values, is_elasti
         grab_axisymmetric_variables(float_variables, values)
     else:
         # single precision variables used within the time loop
-        float_variables['dt'] = values.dt
-        float_variables['c0'] = values.c0
-        float_variables['c_ref'] = values.c_ref
-        float_variables['rho0'] = values.rho0
-        float_variables['rho0_sgx'] = values.rho0_sgx
-        float_variables['rho0_sgy'] = values.rho0_sgy
-        float_variables['rho0_sgz'] = values.rho0_sgz
+        float_variables["dt"] = values.dt
+        float_variables["c0"] = values.c0
+        float_variables["c_ref"] = values.c_ref
+        float_variables["rho0"] = values.rho0
+        float_variables["rho0_sgx"] = values.rho0_sgx
+        float_variables["rho0_sgy"] = values.rho0_sgy
+        float_variables["rho0_sgz"] = values.rho0_sgz
 
 
 def grab_elastic_code_variables(float_variables, kgrid, values):  # pragma: no cover
     # single precision variables used within the time loop
-    float_variables['dt'] = None
-    float_variables['c_ref'] = None
-    float_variables['lambda'] = None
-    float_variables['mu'] = None
+    float_variables["dt"] = None
+    float_variables["c_ref"] = None
+    float_variables["lambda"] = None
+    float_variables["mu"] = None
 
-    float_variables['rho0_sgx'] = None
-    float_variables['rho0_sgy'] = None
-    float_variables['rho0_sgz'] = None
+    float_variables["rho0_sgx"] = None
+    float_variables["rho0_sgy"] = None
+    float_variables["rho0_sgz"] = None
 
-    float_variables['mu_sgxy'] = None
-    float_variables['mu_sgxz'] = None
-    float_variables['mu_sgyz'] = None
+    float_variables["mu_sgxy"] = None
+    float_variables["mu_sgxz"] = None
+    float_variables["mu_sgyz"] = None
 
     # create shift variables used for calculating u_non_staggered and I outputs
     x_shift_neg = np.fft.ifftshift(np.exp(-1j * kgrid.k_vec.x * kgrid.dx / 2))
@@ -167,45 +170,45 @@ def grab_elastic_code_variables(float_variables, kgrid, values):  # pragma: no c
     ddx_k_shift_pos = values.ddx_k_shift_pos
     ddx_k_shift_neg = values.ddx_k_shift_neg
 
-    float_variables['ddx_k_shift_pos_r'] = ddx_k_shift_pos[:Nx_r]
-    float_variables['ddy_k_shift_pos'] = None
-    float_variables['ddz_k_shift_pos'] = None
+    float_variables["ddx_k_shift_pos_r"] = ddx_k_shift_pos[:Nx_r]
+    float_variables["ddy_k_shift_pos"] = None
+    float_variables["ddz_k_shift_pos"] = None
 
-    float_variables['ddx_k_shift_neg_r'] = ddx_k_shift_neg[:Nx_r]
-    float_variables['ddy_k_shift_neg'] = None
-    float_variables['ddz_k_shift_neg'] = None
+    float_variables["ddx_k_shift_neg_r"] = ddx_k_shift_neg[:Nx_r]
+    float_variables["ddy_k_shift_neg"] = None
+    float_variables["ddz_k_shift_neg"] = None
 
-    float_variables['x_shift_neg_r'] = x_shift_neg[:Nx_r]
-    float_variables['y_shift_neg_r'] = y_shift_neg[:Ny_r]
-    float_variables['z_shift_neg_r'] = z_shift_neg[:Nz_r]
+    float_variables["x_shift_neg_r"] = x_shift_neg[:Nx_r]
+    float_variables["y_shift_neg_r"] = y_shift_neg[:Ny_r]
+    float_variables["z_shift_neg_r"] = z_shift_neg[:Nz_r]
 
     del x_shift_neg
 
-    float_variables['pml_x'] = None
-    float_variables['pml_y'] = None
-    float_variables['pml_z'] = None
+    float_variables["pml_x"] = None
+    float_variables["pml_y"] = None
+    float_variables["pml_z"] = None
 
-    float_variables['pml_x_sgx'] = None
-    float_variables['pml_y_sgy'] = None
-    float_variables['pml_z_sgz'] = None
+    float_variables["pml_x_sgx"] = None
+    float_variables["pml_y_sgy"] = None
+    float_variables["pml_z_sgz"] = None
 
-    float_variables['mpml_x_sgx'] = None
-    float_variables['mpml_y_sgy'] = None
-    float_variables['mpml_z_sgz'] = None
+    float_variables["mpml_x_sgx"] = None
+    float_variables["mpml_y_sgy"] = None
+    float_variables["mpml_z_sgz"] = None
 
-    float_variables['mpml_x'] = None
-    float_variables['mpml_y'] = None
-    float_variables['mpml_z'] = None
+    float_variables["mpml_x"] = None
+    float_variables["mpml_y"] = None
+    float_variables["mpml_z"] = None
 
 
 def grab_axisymmetric_variables(float_variables, values):
     # single precision variables used within the time loop
-    float_variables['dt'] = values.dt
-    float_variables['c0'] = values.c0
-    float_variables['c_ref'] = values.c_ref
-    float_variables['rho0'] = values.rho0
-    float_variables['rho0_sgx'] = values.rho0_sgx
-    float_variables['rho0_sgy'] = values.rho0_sgy
+    float_variables["dt"] = values.dt
+    float_variables["c0"] = values.c0
+    float_variables["c_ref"] = values.c_ref
+    float_variables["rho0"] = values.rho0
+    float_variables["rho0_sgx"] = values.rho0_sgx
+    float_variables["rho0_sgy"] = values.rho0_sgy
 
 
 def grab_medium_props(integer_variables, float_variables, medium, is_elastic_code):
@@ -213,7 +216,7 @@ def grab_medium_props(integer_variables, float_variables, medium, is_elastic_cod
     # VARIABLES USED IN NONLINEAR SIMULATIONS
     # =========================================================================
     if medium.is_nonlinear():
-        float_variables['BonA'] = medium.BonA
+        float_variables["BonA"] = medium.BonA
 
     # =========================================================================
     # VARIABLES USED IN ABSORBING SIMULATIONS
@@ -228,19 +231,26 @@ def grab_medium_props(integer_variables, float_variables, medium, is_elastic_cod
     if medium.absorbing:
         if is_elastic_code:  # pragma: no cover
             # add to the variable list
-            float_variables['chi'] = None
-            float_variables['eta'] = None
-            float_variables['eta_sgxy'] = None
-            float_variables['eta_sgxz'] = None
-            float_variables['eta_sgyz'] = None
+            float_variables["chi"] = None
+            float_variables["eta"] = None
+            float_variables["eta_sgxy"] = None
+            float_variables["eta_sgxz"] = None
+            float_variables["eta_sgyz"] = None
         else:
-            float_variables['alpha_coeff'] = medium.alpha_coeff
-            float_variables['alpha_power'] = medium.alpha_power
+            float_variables["alpha_coeff"] = medium.alpha_coeff
+            float_variables["alpha_power"] = medium.alpha_power
 
 
-def grab_source_props(integer_variables, float_variables, source,
-                      u_source_pos_index, s_source_pos_index, p_source_pos_index,
-                      transducer_input_signal, delay_mask):
+def grab_source_props(
+    integer_variables,
+    float_variables,
+    source,
+    u_source_pos_index,
+    s_source_pos_index,
+    p_source_pos_index,
+    transducer_input_signal,
+    delay_mask,
+):
     # =========================================================================
     # SOURCE VARIABLES
     # =========================================================================
@@ -258,11 +268,11 @@ def grab_source_props(integer_variables, float_variables, source,
 
 def grab_velocity_source_props(integer_variables, source, u_source_pos_index):
     # velocity source
-    if any(integer_variables.get(k) for k in ['ux_source_flag', 'uy_source_flag', 'uz_source_flag']):
-        integer_variables['u_source_mode'] = {
-            'dirichlet': 0,
-            'additive-no-correction': 1,
-            'additive': 2,
+    if any(integer_variables.get(k) for k in ["ux_source_flag", "uy_source_flag", "uz_source_flag"]):
+        integer_variables["u_source_mode"] = {
+            "dirichlet": 0,
+            "additive-no-correction": 1,
+            "additive": 2,
         }[source.u_mode]
 
         if integer_variables.ux_source_flag:
@@ -271,16 +281,22 @@ def grab_velocity_source_props(integer_variables, source, u_source_pos_index):
             u_source_many = num_dim2(source.uy) > 1
         elif integer_variables.uz_source_flag:
             u_source_many = num_dim2(source.uz) > 1
-        integer_variables['u_source_many'] = u_source_many
+        integer_variables["u_source_many"] = u_source_many
 
         integer_variables.u_source_index = u_source_pos_index
 
 
 def grab_stress_source_props(integer_variables, source, s_source_pos_index):
     # stress source
-    if integer_variables.sxx_source_flag or integer_variables.syy_source_flag or integer_variables.szz_source_flag \
-            or integer_variables.sxy_source_flag or integer_variables.sxz_source_flag or integer_variables.syz_source_flag:
-        integer_variables.s_source_mode = source.s_mode != 'dirichlet'
+    if (
+        integer_variables.sxx_source_flag
+        or integer_variables.syy_source_flag
+        or integer_variables.szz_source_flag
+        or integer_variables.sxy_source_flag
+        or integer_variables.sxz_source_flag
+        or integer_variables.syz_source_flag
+    ):
+        integer_variables.s_source_mode = source.s_mode != "dirichlet"
         if integer_variables.sxx_source_flag:
             s_source_many = num_dim2(source.sxx) > 1
         elif integer_variables.syy_source_flag:
@@ -301,9 +317,9 @@ def grab_pressure_source_props(integer_variables, source, p_source_pos_index, u_
     # pressure source
     if integer_variables.p_source_flag:
         integer_variables.p_source_mode = {
-            'dirichlet': 0,
-            'additive-no-correction': 1,
-            'additive': 2,
+            "dirichlet": 0,
+            "additive-no-correction": 1,
+            "additive": 2,
         }[source.p_mode]
         integer_variables.p_source_many = num_dim2(source.p) > 1
         integer_variables.p_source_index = p_source_pos_index
@@ -369,7 +385,6 @@ def grab_sensor_props(integer_variables, kgrid_dim, sensor_mask_index, cuboid_co
         integer_variables.sensor_mask_index = sensor_mask_index
 
     elif integer_variables.sensor_mask_type == 1:
-
         cuboid_corners_list = cuboid_corners_list
         # mask is defined as a list of cuboid corners
         if kgrid_dim == 2:
@@ -383,7 +398,7 @@ def grab_sensor_props(integer_variables, kgrid_dim, sensor_mask_index, cuboid_co
         integer_variables.sensor_mask_corners = sensor_mask_corners
 
     else:
-        raise NotImplementedError('unknown option for sensor_mask_type')
+        raise NotImplementedError("unknown option for sensor_mask_type")
 
 
 def grab_nonuniform_grid_props(float_variables, kgrid, is_nonuniform_grid):
@@ -400,39 +415,39 @@ def grab_nonuniform_grid_props(float_variables, kgrid, is_nonuniform_grid):
     dxudxn = kgrid.dudn.x
     if np.array(dxudxn).size == 1:
         dxudxn = np.ones((kgrid.Nx, 1))
-    float_variables['dxudxn'] = dxudxn
+    float_variables["dxudxn"] = dxudxn
 
     dyudyn = kgrid.dudn.y
     if np.array(dyudyn).size == 1:
         dyudyn = np.ones((1, kgrid.Ny))
-    float_variables['dyudyn'] = dyudyn
+    float_variables["dyudyn"] = dyudyn
 
     dzudzn = kgrid.dudn.z
     if np.array(dzudzn).size == 1:
         dzudzn = np.ones((1, 1, kgrid.Nz))
-    float_variables['dzudzn'] = dzudzn
+    float_variables["dzudzn"] = dzudzn
 
     dxudxn_sgx = kgrid.dudn_sg.x
     if np.array(dxudxn).size == 1:
         dxudxn_sgx = np.ones((kgrid.Nx, 1))
-    float_variables['dxudxn_sgx'] = dxudxn_sgx
+    float_variables["dxudxn_sgx"] = dxudxn_sgx
 
     dyudyn_sgy = kgrid.dudn_sg.y
     if np.array(dyudyn).size == 1:
         dyudyn_sgy = np.ones((1, kgrid.Ny))
-    float_variables['dyudyn_sgy'] = dyudyn_sgy
+    float_variables["dyudyn_sgy"] = dyudyn_sgy
 
     dzudzn_sgz = kgrid.dudn_sg.z
     if np.array(dzudzn).size == 1:
         dzudzn_sgz = np.ones((1, 1, kgrid.Nz))
-    float_variables['dzudzn_sgz'] = dzudzn_sgz
+    float_variables["dzudzn_sgz"] = dzudzn_sgz
 
 
 def remove_z_dimension(float_variables, kgrid_dim):
     # remove z-dimension variables for saving 2D files
     if kgrid_dim == 2:
         for k in list(float_variables.keys()):
-            if 'z' in k:
+            if "z" in k:
                 del float_variables[k]
 
 
@@ -442,8 +457,8 @@ def enforce_filename_standards(filepath):
 
     # use .h5 as default if no extension is given
     if len(filename_ext) == 0:
-        filename_ext = '.h5'
-        filepath = filepath + '.h5'
+        filename_ext = ".h5"
+        filepath = filepath + ".h5"
     return filepath, filename_ext
 
 
@@ -451,14 +466,14 @@ def save_file(filepath, integer_variables, float_variables, hdf_compression_leve
     filepath, filename_ext = enforce_filename_standards(filepath)
 
     # save file
-    if filename_ext == '.h5':
+    if filename_ext == ".h5":
         save_h5_file(filepath, integer_variables, float_variables, hdf_compression_level, auto_chunk)
 
-    elif filename_ext == '.mat':
+    elif filename_ext == ".mat":
         save_mat_file(filepath, integer_variables, float_variables)
     else:
         # throw error for unknown filetype
-        raise NotImplementedError('unknown file extension for ''save_to_disk'' filename')
+        raise NotImplementedError("unknown file extension for " "save_to_disk" " filename")
 
 
 def save_h5_file(filepath, integer_variables, float_variables, hdf_compression_level, auto_chunk):
